/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of bgptools. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


/*
 *     Nick Feamster (feamster@lcs.mit.edu)
 *     MIT Lab for Computer Science
 *     November 2001
 */

#include "tabletrace.h"
#define MAX_TRACEROUTE 10

void read_table_file(char *filename, char *tracefile) {

  FILE *fp;
  update upd;
  table_entry te;
  int num_children=0;
  char pfx_m[sizeof(".xxx.xxx.xxx.xxx/xx")];
  char cmd[256];
  pid_t pid;

 
  fp = fopen(filename, "rb");
  if (fp == NULL) { fprintf(stderr, "error: could not open %s\n", filename); };
  
  while(!feof(fp)) {
    read_record(fp, &upd, &te);
  

    sprintf(pfx_m, "%s/%d", te.tpfx, te.tlen);
    te.tpfx[strlen(te.tpfx)-1]='1';

    sprintf(cmd, "%s -- %s %s %s %s >> %s",
	    "/usr/local/bin/bufwrite", "/usr/local/bin/traceroute", pfx_m,
	    te.tpfx, "-n", tracefile);
#ifdef BGPDUMP_DEBUG
    fprintf(stderr, "running \"%s\" \n", cmd);
#endif

    while (num_children > MAX_TRACEROUTE) {
      wait(0);
      num_children--;
    }

    if (!(pid = fork())) {
//        fprintf(stderr, "%s\n", cmd);
      system(cmd);
      _exit(0);
    } else {
      num_children++;
    }

    /* below here, we are the parent */
    
    while (num_children > MAX_TRACEROUTE) {
      wait(0);
      num_children--;
    }
  }
  fclose(fp);
}

int
main (int argc, char **argv) {
  read_table_file(argv[1], argv[2]);
}
