/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of bgptools. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


/*
 *     Nick Feamster (feamster@lcs.mit.edu)
 *     MIT Lab for Computer Science
 *     November 2001
 */

#ifndef __TRACE_H__
#define __TRACE_H__

#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/un.h>
#include <netinet/tcp.h>

/*  #include <async/async.h> */
#include <tcl.h> 
#include "bgp_dump.h"


#define BUF_SZ 1500
#define MAX_TRACEROUTE 50

void _accept_cb(ClientData cd, int mask);
void _read_cb(ClientData cd, int mask);

class Trace {
  FILE *trace_fp;
  char *tracefile;
  int num_children;
  
  int sock, csock;
  struct sockaddr_un servaddr, cliaddr;

  char inbuf[BUF_SZ];
  
  void nonblock(int fd);

 public:
 Trace(char *pathname, char *tracefile);
  ~Trace();

  void accept_cb();
  void read_cb();
  void run_trace(char *prefix, int len);
  
};


class traceClient {
  int csock;
  struct sockaddr_un servaddr;

  char outbuf[BUF_SZ];

 public:
  traceClient(char *pathname);
  ~traceClient();

  int connect_();
  int close_();
  int send(char *buf, int len);
  int sendUpdates(char *filename);
  int sendUpdate(struct update *upd);
  int sendUpdate(char *pkt, int plen);
};


#endif
